package edu.lawrence.canvasdemo

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.border
import androidx.compose.foundation.gestures.detectDragGestures
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material3.Button
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import edu.lawrence.canvasdemo.ui.theme.CanvasDemoTheme

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val graph : GraphModel = GraphModel()
        enableEdgeToEdge()
        setContent {
            CanvasDemoTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    CanvasDemo(graph = graph,modifier = Modifier.padding(innerPadding))
                }
            }
        }
    }
}

@Composable
fun CanvasDemo(graph : GraphModel,modifier: Modifier = Modifier) {
    val vertices = remember { graph.vertices }
    val drawingEdge = remember { mutableStateOf(false)}
    val startOffset = remember { mutableStateOf(Offset.Zero) }
    val curOffset = remember { mutableStateOf(Offset.Zero) }

    fun startDrag(offset: Offset) {
        startOffset.value = offset
        curOffset.value = offset
        if(drawingEdge.value) {
            graph.startDraw(offset)
        } else {
            graph.startDrag(offset)
        }
    }

    fun continueDrag(dragAmount:Offset) {
        curOffset.value = Offset(x = curOffset.value.x + dragAmount.x,y = curOffset.value.y + dragAmount.y)
        if(!drawingEdge.value) {
            graph.continueDrag(curOffset.value)
        }
    }

    fun endDrag() {
        if(drawingEdge.value) {
            graph.endDraw(curOffset.value)
        } else {
            graph.endDrag(curOffset.value)
        }
        startOffset.value = Offset.Zero
        curOffset.value = Offset.Zero
    }

    Column(verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally, modifier = modifier) {
        Spacer(modifier = Modifier.height(20.dp))
        Row(
            horizontalArrangement = Arrangement.Start,
            verticalAlignment = Alignment.CenterVertically
        ) {
            Text("Draw Edges:")
            Spacer(modifier = Modifier.width(10.dp))
            Switch(
                checked = drawingEdge.value,
                onCheckedChange = { drawingEdge.value = it })
        }
        Canvas(modifier = Modifier.fillMaxWidth().height(400.dp).padding(10.dp).border(BorderStroke(2.dp, Color.Blue))
            .pointerInput(Unit) {
                detectTapGestures(onPress = { startDrag(it) })
            }
            .pointerInput(Unit){
                detectDragGestures(onDrag = {_,amount->continueDrag(amount)},
                    onDragEnd = {endDrag()})
            }) {
            for( edge in graph.edges) {
                edge.drawIn(this)
            }
            for( vertex in graph.vertices) {
                vertex.drawIn(this)
            }
            if(startOffset.value != curOffset.value && drawingEdge.value) {
                drawLine(
                    color = Color.Black,
                    start = startOffset.value,
                    end = curOffset.value,
                    strokeWidth = 1.0f
                )
            }
        }
        Button(onClick = { graph.makeVertex(Offset(x = 600f,y = 600f)) }) {
            Text("Make New Vertex")
        }
    }
}


@Preview(showBackground = true)
@Composable
fun GreetingPreview() {
    val graph = GraphModel()
    CanvasDemoTheme {
        CanvasDemo(graph)
    }
}